#### Setup ####
## Load packages
library(here)
library(readr)
library(dplyr)
library(datawrangling)
library(englelab)

## Set Import/Output Directories
import_dir <- "Data Files/Raw Data/Merged"
output_dir <- "Data Files/Raw Data"

## Set Import/Output Filenames
task <- "Auditory_SimonDL"
import_file <- paste(task, ".txt", sep = "")
output_file <- paste(task, "raw.csv", sep = "_")
################

## Completed Subjects ####
subjlist <- read_csv(here("Data Files", "subjlist_completed.csv"))
##########################

#### Import ####
data_import <- read_delim(here(import_dir, import_file), "\t", 
                          escape_double = FALSE, trim_ws = TRUE,
                          guess_max = 10000) %>%
duplicates_check(id = "Subject", unique = c("SessionDate", "SessionTime"),
                 n = 1, remove = TRUE,
                 save_as = here(output_dir, "duplicates", 
                                paste(task, "_duplicates.csv", sep = ""))) %>%
  filter(Subject %in% subjlist$Subject)
################

#### Tidy raw data ####
data_raw <- data_import %>%
  rename(TrialProc = `Procedure[Trial]`, 
         CorrectResponseXReal = `CorrectResponse[SubTrial]`,
         CorrectResponseXPractice = `CorrectResponse[Trial]`,
         SimonDurationXReal = `SimonDuration[SubTrial]`,
         SimonDurationXPractice = `SimonDuration[Trial]`,
         RealPanValue = `PanValue[SubTrial]`,
         PracticePanValue = `PanValue[Trial]`)%>%
  filter(TrialProc != "RestProc") %>%
  mutate(TrialProc = case_when(TrialProc == "ASimonPractice2" ~ "practice",
                               TrialProc == "BlockProc" ~ "real"),
         Accuracy = 
           case_when(TrialProc == "practice" ~ PracticeTest.ACC,
                     TrialProc == "real" ~ Test.ACC),
         Response = 
           case_when(TrialProc == "practice" ~ PracticeTest.RESP,
                     TrialProc == "real" ~ Test.RESP),
         
         Response = case_when(Response == "q" ~ "left",
                              Response == "p" ~ "right"),
         MissedDL.RESP = case_when(TrialProc == "practice" ~ PracticeMissedDL.RESP,
                                   TrialProc == "real" ~ MissedDL.RESP),
         Response = case_when(is.na(Response) & MissedDL.RESP == "q" ~ "left",
                              is.na(Response) & MissedDL.RESP == "p" ~ "right",
                              TRUE ~ Response),
         CorrectResponse = case_when(CorrectResponseXReal == "q" ~ "left",
                                     CorrectResponseXReal == "p" ~ "right",
                                     CorrectResponseXPractice == "q" ~ "left",
                                     CorrectResponseXPractice == "p" ~ "right"),
         
         RT = 
           case_when(TrialProc == "practice" ~ PracticeTest.RT,
                     TrialProc == "real" ~ Test.RT),
         Trial = 
           case_when(TrialProc == "practice" ~ PracticeList2.Sample,
                     TrialProc == "real" ~ TrialCount,
                     TRUE ~ Trial),
         RealPanValue = case_when(TrialProc == "practice" ~ PracticePanValue,
                                  TrialProc == "real" ~ RealPanValue),
         RealPanValue = case_when(RealPanValue == -10000 ~ "left ear",
                                  RealPanValue == 10000 ~ "right ear"),
         ResponseDeadline = 
           case_when(SimonDurationXReal == "(infinite)" ~ as.integer(NA),
                     TRUE ~ as.integer(SimonDurationXReal)),
         PracticeResponseDeadline = 
           case_when(SimonDurationXPractice == "(infinite)" ~ as.integer(NA),
                     TRUE ~ as.integer(SimonDurationXPractice)),
         ResponseDeadline = case_when(TrialProc == "real" ~ ResponseDeadline,
                                      TrialProc == "practice" ~ PracticeResponseDeadline),
         TrialType = case_when(TrialProc == "real" ~ TrialType,
                               TrialProc == "practice" ~ PracticeTrialType),
         InstructionsTime = InstructionsTime / 60000,
         PracticeTime = PracticeTime / 60000,
         TaskTime = TaskTime / 60000,
         AdminTime = AdminTime / 60000) %>%
  select(Subject, TrialProc, Trial, TrialType, ResponseDeadline,
         Accuracy, RT, Response, CorrectResponse, ReversalMade, ReversalNumber = ReversalNum,
         RealPanValue, SimonDLTime, SimonDLTotalAccuracy, SimonDLTotalRT, SimonDLCorrectRT, 
         SimonMissedDeadlines, AdminTime, SessionDate, SessionTime)
#######################

#### Output ####
write_csv(data_raw, here(output_dir, output_file))
################

rm(list=ls())
