#### Setup ####
## Load Packages
library(here)
library(readr)
library(dplyr)
library(tidyr)
library(psych)      # for cronbach's alpha
library(knitr)      # for log report at end
library(kableExtra) # for log report at end

## Load Functions
source(here("R Scripts", "_score_adaptivethreshold.R"))
source(here("R Scripts", "_print_log.R"))
source(here("R Scripts", "_outliers.R"))

## Set Import/Output Directories
import_dir <- "Data Files/Raw Data"
output_dir <- "Data Files/Scored Data"

## Set Import/Output Filenames
task <- "Auditory_StroopDL"
import_file <- paste(task, "raw.csv", sep = "_")
output_file <- paste(task, "Scores.csv", sep = "_")

## Set Data Cleaning Params
acc_criteria <- .5
outlier_criteria <- 3.5
rt_criteria <- 200
###############

#### Import ####
data_import <- read_csv(here(import_dir, import_file)) %>%
  filter(TrialProc == "real")
################

#### Score Data ####
scores_last4rev <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL", 
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "Last 4 Reversals")

scores_last8rev <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL",
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "Last 8 Reversals")

scores_last10rev <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL",
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "Last 10 Reversals")

scores_lasttrial <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL",
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "Last Trial")

scores_allrev <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL",
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "All Reversals")

scores_overallmedian <- data_import %>%
  filter(TrialType == "Incongruent") %>%
  group_by(Subject) %>%
  score_adaptivethreshold(taskname = "Auditory_StroopDL",
                          adaptive_variable = "ResponseDeadline",
                          threshold_method = "Overall Median")

scores_condition_perf <- data_import %>%
  group_by(Subject, TrialType) %>%
  mutate(RT = ifelse(Accuracy == 0, NA, RT),
         RT = ifelse(RT < rt_criteria, NA, RT),
         MissedDeadline = ifelse(RT <= ResponseDeadline, 0, 1)) %>%
  summarise(ACC = mean(Accuracy, na.rm = TRUE),
            RT = mean(RT, na.rm = TRUE),
            MissedDeadlines = mean(MissedDeadline)) %>%
  pivot_wider(id_cols = "Subject",
              names_from = "TrialType",
              values_from = c("ACC", "RT", "MissedDeadlines"),
              names_glue = "{TrialType}_{.value}") %>%
  rename_with(~ paste("Auditory_StroopDL_", .x, sep = ""), contains("congruent"))

scores_overall_perf <- data_import %>%
  group_by(Subject) %>%
  mutate(RT = ifelse(Accuracy == 0, NA, RT),
         RT = ifelse(RT < rt_criteria, NA, RT),
         MissedDeadline = ifelse(RT <= ResponseDeadline, 0, 1)) %>%
  summarise(ACC = mean(Accuracy, na.rm = TRUE),
            RT = mean(RT, na.rm = TRUE),
            MissedDeadlines = mean(MissedDeadline),
            AdminTime = mean(AdminTime)) %>%
  rename_with(~ paste("Auditory_StroopDL_Overall_", .x, sep = ""), 
              c(ACC, RT, MissedDeadlines, AdminTime))

data_scores <- merge(scores_last4rev, scores_last8rev, by = "Subject") %>%
  merge(scores_last10rev, by = "Subject") %>%
  merge(scores_lasttrial, by = "Subject") %>%
  merge(scores_allrev, by = "Subject") %>%
  merge(scores_overallmedian, by = "Subject") %>%
  merge(scores_condition_perf, by = "Subject") %>%
  merge(scores_overall_perf, by = "Subject")
####################

#### Clean Data ####
data_remove <- data_scores %>%
  filter(Auditory_StroopDL_Overall_ACC <= acc_criteria) 

data_scores <- filter(data_scores, !(Subject %in% data_remove$Subject))

data_outliers_1 <- data_scores %>%
  outliers("Auditory_StroopDL_Last4Rev.ResponseDeadline", outlier_criteria)

data_scores <- filter(data_scores, !(Subject %in% data_outliers_1$Subject))

data_outliers_2 <- data_scores %>%
  outliers("Auditory_StroopDL_Last4Rev.ResponseDeadline", outlier_criteria)

data_scores <- filter(data_scores, !(Subject %in% data_outliers_2$Subject))
####################

#### Calculate Reliability ####
reliability <- data_import %>%
  filter(Subject %in% data_scores$Subject) %>%
  group_by(Subject) %>%
  mutate(Trial_New = as.numeric(row_number()),
         var_last_reversals = 
           case_when(
             ReversalNumber >= max(ReversalNumber, na.rm = TRUE) - (4 - 1) ~ 
               ResponseDeadline, 
             TRUE ~ as.numeric(NA)),
         last_reversals_firsttrial = 
           case_when(
             ReversalNumber == max(ReversalNumber, na.rm = TRUE) - (4 - 1) ~ 
               Trial_New, 
             TRUE ~ as.numeric(NA)),
         last_reversals_lasttrial = 
           case_when(ReversalNumber == max(ReversalNumber, na.rm = TRUE) ~
                       Trial_New,
                     TRUE ~ as.numeric(NA))) %>%
  filter(!is.na(var_last_reversals))

cronbachalpha <- reliability %>%
  group_by(Subject) %>%
  mutate(Trial = row_number()) %>%
  pivot_wider(id_cols = "Subject",
              names_from = "Trial",
              values_from = "var_last_reversals") %>%
  ungroup() %>%
  psych::alpha()

data_scores$StroopDL_Last4Rev.ResponseDeadline.cronbachalpha <- 
  cronbachalpha$total$std.alpha
###############################

#### Output ####
write_csv(data_scores, here(output_dir, output_file))
################

## Log Report ####
print_text(data_import, data_remove, data_outliers_1, data_outliers_2, data_scores)
print_removed(data_remove)
print_outliers_1(data_outliers_1)
print_outliers_2(data_outliers_2)
##############

rm(list=ls())


